# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import csv
import sys
import os
import os.path as osp
import glob

import nibabel as nb

from cubicweb.dataimport import SQLGenObjectStore as Store

from cubes.brainomics.importers.helpers import get_image_info


SEX = {'1': u'male', '2': u'female'}
HANDEDNESS = {'R': u'right', 'L': u'left', 'Ambi': u'ambidextrous', 'Mixed': u'mixed'}


###############################################################################
### GROUPS AND SCORES #########################################################
###############################################################################
def create_groups_and_categories(session):
    """Create groups and categories"""
    groups = {}
    groups[1] = store.create_entity('Group', identifier=u'abide_autism', name=u'Diagnostic group Autism').eid
    groups[2] = store.create_entity('Group', identifier=u'abide_control', name=u'Diagnostic group Control').eid
    categories = {}
    categories['0'] = store.create_entity('Group', identifier=u'abide_category_control',
                                          name=u'Diagnostic category Control').eid
    categories['1'] = store.create_entity('Group', identifier=u'abide_category_autism',
                                          name=u'Diagnostic category Autism').eid
    categories['2'] = store.create_entity('Group', identifier=u'abide_category_asperger',
                                          name=u'Diagnostic category Aspergers').eid
    categories['3'] = store.create_entity('Group', identifier=u'abide_category_pdd_nos',
                                          name=u'Diagnostic category PDD-NOS').eid
    categories['4'] = store.create_entity('Group', identifier=u'abide_category_pdd_nos_asperger',
                                          name=u'Diagnostic category Aspergers or PDD-NOS').eid
    categories['u'] = store.create_entity('Group', identifier=u'abide_category_unknown',
                                          name=u'Diagnostic category unknown').eid
    return groups, categories

def create_score_defs(store):
    score_defs = {}
    # Various scores
    score_defs['dx_group'] = store.create_entity('ScoreDefinition', name=u'Diagnostic Group',
                                                  category=u'biological', type=u'numerical',
                                                  possible_values=u'1 = Autism; 2 = Control').eid
    score_defs['dsm_iv_tr'] = store.create_entity('ScoreDefinition', name=u'DSM-IV-TR Diagnostic Category',
                                                  category=u'biological', type=u'numerical',
                                                  possible_values=u'0 = Control; 1 = Autism; 2 = Aspergers; 3 = PDD-NOS; 4 = Aspergers or PDD-NOS').eid
    score_defs['handedness_score'] = store.create_entity('ScoreDefinition', name=u'Subject Handedness Score',
                                                         category=u'biological', type=u'numerical',
                                                         possible_values=u'Positive number = Right;\
                                                         Negative number = Left; 0 = Ambidextrous').eid
    score_defs['scq_total'] = store.create_entity('ScoreDefinition', name=u'Social Communication Questionnaire Total',
                                                  category=u'behavioral', type=u'numerical',
                                                  possible_values=u'0-39').eid
    score_defs['aq_total'] = store.create_entity('ScoreDefinition', name=u'Total Raw Score of the Autism Quotient',
                                                  category=u'behavioral', type=u'numerical',
                                                  possible_values=u'0-50').eid
    score_defs['comorbidity'] = store.create_entity('ScoreDefinition', name=u'Any other comorbidities ?',
                                                    category=u'biological', type=u'string').eid
    score_defs['current_med_status'] = store.create_entity('ScoreDefinition', name=u'Currently Taking Medications ?',
                                                           category=u'biological', type=u'numerical',
                                                           possible_values=u'0 = not taking medication; 1 = taking medication').eid
    score_defs['medication_name'] = store.create_entity('ScoreDefinition', name=u'Active ingredient of any current psychoactive medications',
                                                        category=u'biological', type=u'string').eid
    score_defs['off_stimulants_at_scan'] = store.create_entity('ScoreDefinition', name=u'Off stimulants 24 hours prior to scan ?',
                                                               category=u'biological', type=u'numerical',
                                                               possible_values=u'0 = no; 1 = yes').eid
    score_defs['eye_status_at_scan'] = store.create_entity('ScoreDefinition', name=u'Eye Status During Rest Scan',
                                                               category=u'behavioral', type=u'numerical',
                                                               possible_values=u'1 = open; 2 = closed').eid
    score_defs['age_at_mprage'] = store.create_entity('ScoreDefinition', name=u'Age at Anatomical Scan in years',
                                                               category=u'biological', type=u'numerical').eid
    score_defs['bmi'] = store.create_entity('ScoreDefinition', name=u'Body Mass Index',
                                            category=u'biological', type=u'numerical').eid

    # FIQ/VIQ/PIQ scores
    for test_type in (u'DAS_II_SA', u'GIT', u'HAWIK', u'WAIS_III', u'WASI',
                      # XXX WAIS/WISC are not defined in the specs...
                      u'WAIS', u'WISC', u'WISC_IV_FULL', u'HAWIK_IV',
                      u'WISC_III', u'WISC_III_DUTCH', u'WISC_IV_4_SUBTESTS',
                      u'WISCC_IV_FULL', u'WST'):
        name = u'FIQ Standard Score - %s' % test_type
        score_defs['fiq_%s' % test_type.lower()] = store.create_entity('ScoreDefinition', name=name,
                                                                       category=u'FIQ',
                                                                       type=u'numerical').eid
    for test_type in (u'DAS_II_SA', u'GIT', u'HAWIK', u'PPVT', u'WAIS_III', u'WASI',
                      # XXX WAIS/WISC are not defined in the specs...
                      u'WAIS', u'WISC', u'WISC_IV_FULL',
                      u'WISC_III', u'WISC_III_DUTCH', u'WISC_IV_4_SUBTESTS',
                      u'WISCC_IV_FULL', u'WST'):
        name = u'VIQ Standard Score - %s' % test_type
        score_defs['viq_%s' % test_type.lower()] = store.create_entity('ScoreDefinition', name=name,
                                                                       category=u'VIQ',
                                                                       type=u'numerical').eid
    for test_type in (u'DAS_II_SA', u'GIT', u'HAWIK', u'RAVENS', u'WAIS_III', u'WASI',
                      # XXX WAIS/WISC are not defined in the specs...
                      u'WAIS', u'WISC', u'WISC_IV_FULL',
                      u'WISC_III', u'WISC_III_DUTCH', u'WISC_IV_4_SUBTESTS',
                      u'WISCC_IV_FULL', u'WST'):
        name = u'PIQ Standard Score - %s' % test_type
        score_defs['piq_%s' % test_type.lower()] = store.create_entity('ScoreDefinition', name=name,
                                                                       category=u'PIQ',
                                                                       type=u'numerical').eid
    return score_defs


###############################################################################
### QUESTIONNAIRES ############################################################
###############################################################################
def create_adi_questionnaire(store):
    """Create a questionnaire for adi"""
    questionnaire_eid = store.create_entity('Questionnaire',
                                            name=u'Autism Diagnostic Interview-Revised (ADI)',
                                            identifier=u'abide_adi', type=u'behavioral').eid
    questions = {}
    text = u'Reciprocal Social Interaction Subscore (A) Total for Autism Diagnostic Interview-Revised'
    questions['adi_r_social_total_a'] = store.create_entity('Question',
                                                            identifier=u'adi_r_social_total_a',
                                                            position=0, text=text,
                                                            type=u'numerical',
                                                            questionnaire=questionnaire_eid,
                                                            possible_answers=u'0-30').eid
    text = u'Abnormalitites in Communication Subscore (B) Total for Autism Diagnostic Interview-Revised',
    questions['adi_r_verbal_total_bv'] = store.create_entity('Question',
                                                            identifier=u'adi_r_verbal_total_bv',
                                                            position=1, text=text,
                                                            type=u'numerical',
                                                            questionnaire=questionnaire_eid,
                                                            possible_answers=u'0-26').eid
    text = u'Restricted, Repetitive and Stereotyped Patters of Behavior Subscore (C) Total for Autism Diagnostic Interview-Revised'
    questions['adi_r_rrb_total_c'] = store.create_entity('Question',
                                                         identifier=u'adi_r_rrb_total_c',
                                                         position=2, text=text,
                                                         type=u'numerical',
                                                         questionnaire=questionnaire_eid,
                                                         possible_answers=u'0-12').eid
    text = u'Abnormality of Development Evident at or Before 36 Monts Subscore (D) Total for Autism Diagnostic Interview-Revised'
    questions['adi_r_onset_total_d'] = store.create_entity('Question',
                                                         identifier=u'adi_r_onset_total_d',
                                                         position=3, text=text,
                                                         type=u'numerical',
                                                         questionnaire=questionnaire_eid,
                                                         possible_answers=u'0-5').eid
    text = u'Was ADI scored and administered by research reliable personnel?'
    questions['adi_r_rsrch_reliable'] = store.create_entity('Question',
                                                         identifier=u'adi_r_rsrch_reliable',
                                                         position=4, text=text,
                                                         type=u'boolean',
                                                         questionnaire=questionnaire_eid,
                                                         possible_answers=u'0 = not research reliable; 1 = research reliable').eid
    return questionnaire_eid, questions

def create_ados_questionnaire(store):
    """Create a questionnaire for ados"""
    questionnaire_eid = store.create_entity('Questionnaire',
                                            name=u'Autism Diagnostic Observation Schedule Module (ADOS)',
                                            identifier=u'abide_ados', type=u'behavioral').eid
    questions = {}
    text = u'Autism Diagnostic Observation Schedule Module'
    questions['ados_module'] = store.create_entity('Question',
                                                   identifier=u'ados_module',
                                                   position=0, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-4').eid
    text = u'Classical Total ADOS Score (Communication subscore + Social Interaction subscore)'
    questions['ados_total'] = store.create_entity('Question',
                                                   identifier=u'ados_total',
                                                   position=1, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'0-22').eid
    text = u'Communication Total Subscore of the Classic ADOS'
    questions['ados_comm'] = store.create_entity('Question',
                                                 identifier=u'ados_comm',
                                                 position=2, text=text,
                                                 type=u'numerical',
                                                 questionnaire=questionnaire_eid,
                                                 possible_answers=u'0-8').eid
    text = u'Social Total Subscore of the Classic ADOS'
    questions['ados_social'] = store.create_entity('Question',
                                                 identifier=u'ados_social',
                                                 position=3, text=text,
                                                 type=u'numerical',
                                                 questionnaire=questionnaire_eid,
                                                 possible_answers=u'0-14').eid
    text = u'Stereotyped Behaviors and Restricted Interests Total Subscore of the Classic ADOS'
    questions['ados_stereo_behav'] = store.create_entity('Question',
                                                 identifier=u'ados_stereo_behav',
                                                 position=4, text=text,
                                                 type=u'numerical',
                                                 questionnaire=questionnaire_eid,
                                                 possible_answers=u'0-8').eid
    text = u'Was ADOS scored and administered by research reliable personnel?'
    questions['ados_rsrch_reliable'] = store.create_entity('Question',
                                                           identifier=u'ados_rsrch_reliable',
                                                           position=5, text=text,
                                                           type=u'boolean',
                                                           questionnaire=questionnaire_eid,
                                                           possible_answers=u'0 = not research reliable; 1 = research reliable').eid
    text = u'Social Affect Total Subscore for Gotham Algorithm of the ADOS'
    questions['ados_gotham_soc_affect'] = store.create_entity('Question',
                                                              identifier=u'ados_gotham_soc_affect',
                                                              position=6, text=text,
                                                              type=u'numerical',
                                                              questionnaire=questionnaire_eid,
                                                              possible_answers=u'0-20').eid
    text = u'Restrictive and Repetitive Behaviors Total Subscore for Gotham Algorithm of the ADOS'
    questions['ados_gotham_rrb'] = store.create_entity('Question',
                                                         identifier=u'ados_gotham_rrb',
                                                         position=7, text=text,
                                                         type=u'numerical',
                                                         questionnaire=questionnaire_eid,
                                                         possible_answers=u'0-8').eid
    text = u'Social Affect Total + Restricted and Repetitive Behaviors Total'
    questions['ados_gotham_total'] = store.create_entity('Question',
                                                         identifier=u'ados_gotham_total',
                                                         position=8, text=text,
                                                         type=u'numerical',
                                                         questionnaire=questionnaire_eid,
                                                         possible_answers=u'0-28').eid
    text = u'Individually Calibrated Severity Score for Gotham Algorithm of the ADOS'
    questions['ados_gotham_severity'] = store.create_entity('Question',
                                                         identifier=u'ados_gotham_severity',
                                                         position=9, text=text,
                                                         type=u'numerical',
                                                         questionnaire=questionnaire_eid,
                                                         possible_answers=u'1-10').eid
    return questionnaire_eid, questions

def create_srs_questionnaire(store):
    """Create a questionnaire for srs"""
    questionnaire_eid = store.create_entity('Questionnaire',
                                            name=u'Social Responsiveness Scale Version',
                                            identifier=u'avide_srs', type=u'behavioral').eid
    questions = {}
    text = u'Social Responsiveness Scale Version'
    questions['srs_version'] = store.create_entity('Question',
                                                   identifier=u'srs_version',
                                                   position=0, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1 = child; 2 = adult').eid
    text = u'Total Raw Score the Social Responsiveness Scale'
    questions['srs_raw_total'] = store.create_entity('Question',
                                                   identifier=u'srs_raw_total',
                                                   position=1, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'0 >= 117').eid
    text = u'Social Responsiveness Scale Social Avareness Subscore Raw Total'
    questions['srs_awareness'] = store.create_entity('Question',
                                                   identifier=u'srs_awareness',
                                                   position=2, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'0 >= 19').eid
    text = u'Social Responsiveness Scale Social Cognition Subscore Raw Total'
    questions['srs_cognition'] = store.create_entity('Question',
                                                     identifier=u'srs_cognition',
                                                     position=3, text=text,
                                                     type=u'numerical',
                                                     questionnaire=questionnaire_eid,
                                                     possible_answers=u'0 >= 24').eid
    text = u'Social Responsiveness Scale Social Communication Subscore Raw Total'
    questions['srs_communication'] = store.create_entity('Question',
                                                     identifier=u'srs_communication',
                                                     position=4, text=text,
                                                     type=u'numerical',
                                                     questionnaire=questionnaire_eid,
                                                     possible_answers=u'0 >= 43').eid
    text = u'Social Responsiveness Scale Social Motivation Subscore Raw Total'
    questions['srs_motivation'] = store.create_entity('Question',
                                                     identifier=u'srs_motivation',
                                                     position=5, text=text,
                                                     type=u'numerical',
                                                     questionnaire=questionnaire_eid,
                                                     possible_answers=u'0 >= 22').eid
    text = u'Social Responsiveness Scale Social Autistic Mannerisms Subscore Raw Total'
    questions['srs_mannerisms'] = store.create_entity('Question',
                                                     identifier=u'srs_mannerisms',
                                                     position=6, text=text,
                                                     type=u'numerical',
                                                     questionnaire=questionnaire_eid,
                                                     possible_answers=u'0 >= 22').eid
    return questionnaire_eid, questions

def create_vineland_questionnaire(store):
    """Create a questionnaire for vineland"""
    questionnaire_eid = store.create_entity('Questionnaire',
                                            name=u'Vineland Adaptive Behavior Scales',
                                            identifier=u'avide_vineland', type=u'behavioral').eid
    questions = {}
    text = u'Vineland Adaptive Behavior Scales Receptive Language V Scaled Score'
    questions['vineland_receptive_v_scaled'] = store.create_entity('Question',
                                                                   identifier=u'vineland_receptive_v_scaled',
                                                                   position=0, text=text,
                                                                   type=u'numerical',
                                                                   questionnaire=questionnaire_eid,
                                                                   possible_answers=u'1-24').eid
    text = u'Vineland Adaptive Behavior Scales Expressive Language V Scaled Score'
    questions['vineland_expressive_v_scaled'] = store.create_entity('Question',
                                                                   identifier=u'vineland_expressive_v_scaled',
                                                                   position=1, text=text,
                                                                   type=u'numerical',
                                                                   questionnaire=questionnaire_eid,
                                                                   possible_answers=u'1-24').eid
    text = u'Vineland Adaptive Behavior Scales Written Language V Scaled Score'
    questions['vineland_written_v_scaled'] = store.create_entity('Question',
                                                                   identifier=u'vineland_written_v_scaled',
                                                                   position=2, text=text,
                                                                   type=u'numerical',
                                                                   questionnaire=questionnaire_eid,
                                                                   possible_answers=u'1-24').eid
    text = u'Vineland Adaptive Behavior Scales Communication Standard Score'
    questions['vineland_communication_standard'] = store.create_entity('Question',
                                                                   identifier=u'vineland_communication_standard',
                                                                   position=3, text=text,
                                                                   type=u'numerical',
                                                                   questionnaire=questionnaire_eid,
                                                                   possible_answers=u'20-160').eid
    text = u'Vineland Adaptive Behavior Scales Personal Daily Living Skills V Scaled Score'
    questions['vineland_personal_v_scaled'] = store.create_entity('Question',
                                                                   identifier=u'vineland_personal_v_scaled',
                                                                   position=4, text=text,
                                                                   type=u'numerical',
                                                                   questionnaire=questionnaire_eid,
                                                                   possible_answers=u'1-24').eid
    text = u'Vineland Adaptive Behavior Scales Domestic Daily Living Skills V Scaled Score'
    questions['vineland_domestic_v_scaled'] = store.create_entity('Question',
                                                                   identifier=u'vineland_domestic_v_scaled',
                                                                   position=5, text=text,
                                                                   type=u'numerical',
                                                                   questionnaire=questionnaire_eid,
                                                                   possible_answers=u'1-24').eid
    text = u'Vineland Adaptive Behavior Scales Community Daily Living Skills V Scaled Score'
    questions['vineland_community_v_scaled'] = store.create_entity('Question',
                                                                   identifier=u'vineland_community_v_scaled',
                                                                   position=6, text=text,
                                                                   type=u'numerical',
                                                                   questionnaire=questionnaire_eid,
                                                                   possible_answers=u'1-24').eid
    text = u'Vineland Adaptive Behavior Scales Daily Living Skills Standard Score'
    questions['vineland_dailylvng_standard'] = store.create_entity('Question',
                                                                   identifier=u'vineland_dailylvng_standard',
                                                                   position=7, text=text,
                                                                   type=u'numerical',
                                                                   questionnaire=questionnaire_eid,
                                                                   possible_answers=u'20-160').eid
    text = u'Vineland Adaptive Behavior Scales Interpersonal Relationships V Scaled Score'
    questions['vineland_interpersonal_v_scaled'] = store.create_entity('Question',
                                                                       identifier=u'vineland_interpersonal_v_scaled',
                                                                       position=8, text=text,
                                                                       type=u'numerical',
                                                                       questionnaire=questionnaire_eid,
                                                                       possible_answers=u'1-24').eid
    text = u'Vineland Adaptive Behavior Scales Play and Leisure Time V Scaled Score'
    questions['vineland_play_v_scaled'] = store.create_entity('Question',
                                                             identifier=u'vineland_play_v_scaled',
                                                             position=9, text=text,
                                                             type=u'numerical',
                                                             questionnaire=questionnaire_eid,
                                                             possible_answers=u'1-24').eid
    text = u'Vineland Adaptive Behavior ScalesCoping Skills V Scaled Score'
    questions['vineland_coping_v_scaled'] = store.create_entity('Question',
                                                             identifier=u'vineland_coping_v_scaled',
                                                             position=10, text=text,
                                                             type=u'numerical',
                                                             questionnaire=questionnaire_eid,
                                                             possible_answers=u'1-24').eid
    text = u'Vineland Adaptive Behavior Scales Socialization Standard Score'
    questions['vineland_social_standard'] = store.create_entity('Question',
                                                                   identifier=u'vineland_social_standard',
                                                                   position=11, text=text,
                                                                   type=u'numerical',
                                                                   questionnaire=questionnaire_eid,
                                                                   possible_answers=u'20-160').eid
    text = u'Sum of Vineland Standard Scores (Communication + Daily Living Skills + Socialization)'
    questions['vineland_sum_scores'] = store.create_entity('Question',
                                                           identifier=u'vineland_sum_scores',
                                                           position=12, text=text,
                                                           type=u'numerical',
                                                           questionnaire=questionnaire_eid,
                                                           possible_answers=u'76-480').eid
    text = u'Vineland Adaptive Behavior Composite Standard score'
    questions['vineland_abc_standard'] = store.create_entity('Question',
                                                           identifier=u'vineland_abc_standard',
                                                           position=13, text=text,
                                                           type=u'numerical',
                                                           questionnaire=questionnaire_eid,
                                                           possible_answers=u'20-160').eid
    text = u'Vineland Adaptive Behavior Scales Informant'
    questions['vineland_informant'] = store.create_entity('Question',
                                                           identifier=u'vineland_informant',
                                                           position=14, text=text,
                                                           type=u'numerical',
                                                           questionnaire=questionnaire_eid,
                                                           possible_answers=u'1 = parent; 2 = self').eid
    return questionnaire_eid, questions

def create_wisc_iv_questionnaire(store):
    """Create a questionnaire for wisc_iv"""
    questionnaire_eid = store.create_entity('Questionnaire', name=u'WISC-IV',
                                            identifier=u'avide_wisc_iv', type=u'behavioral').eid
    questions = {}
    text = u'WISC-IV Verbal Comprehension Index'
    questions['wisc_iv_vci'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_vci',
                                                   position=0, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'3-57').eid
    text = u'WISC-IV Perceptual Reasoning Index'
    questions['wisc_iv_pri'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_pri',
                                                   position=1, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'3-57').eid
    text = u'WISC-IV Working Memory Index'
    questions['wisc_iv_wmi'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_wmi',
                                                   position=2, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'2-38').eid
    text = u'WISC-IV Processing Speed Index'
    questions['wisc_iv_psi'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_psi',
                                                   position=3, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'2-38').eid
    text = u'WISC-IV Sim Scaled'
    questions['wisc_iv_sim_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_sim_scaled',
                                                   position=4, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    text = u'WISC-IV Vocabulary Scaled'
    questions['wisc_iv_vocab_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_vocab_scaled',
                                                   position=5, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    text = u'WISC-IV Information Scaled'
    questions['wisc_iv_info_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_info_scaled',
                                                   position=6, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    text = u'WISC-IV Block Design Scaled'
    questions['wisc_iv_blk_dsn_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_blk_dsn_scaled',
                                                   position=7, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    text = u'WISC-IV Picture Concepts Scaled'
    questions['wisc_iv_pic_con_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_pic_con_scaled',
                                                   position=8, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    text = u'WISC-IV Matrix Reasoning Scaled'
    questions['wisc_iv_matrix_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_matrix_scaled',
                                                   position=9, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    text = u'WISC-IV Digit Span Scaled'
    questions['wisc_iv_digit_span_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_digit_span_scaled',
                                                   position=10, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    text = u'WISC-IV Letter-Number Sequencing Scaled'
    questions['wisc_iv_let_num_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_let_num_scaled',
                                                   position=11, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    text = u'WISC-IV Coding Scaled'
    questions['wisc_iv_coding_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_coding_scaled',
                                                   position=12, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    text = u'WISC-IV Symbol Search Scaled'
    questions['wisc_iv_sym_scaled'] = store.create_entity('Question',
                                                   identifier=u'wisc_iv_sym_scaled',
                                                   position=13, text=text,
                                                   type=u'numerical',
                                                   questionnaire=questionnaire_eid,
                                                   possible_answers=u'1-19').eid
    return questionnaire_eid, questions


def import_questionnaire(store, subject_eid, center_eid, studies_eids, questionnaire_eid,
                         questions, answers_listing, subj_id, label, valid_ind, user_ident=None):
    assessment = store.create_entity('Assessment', identifier=u'%s_%s' % (label, subj_id),
                                     # XXX datetime, timepoint
                                     protocol=u'ABIDE')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    if valid_ind:
        valid = bool(int(line[valid_ind])) if line[valid_ind] else False
    else:
        valid = True
    if user_ident:
        user_ident = u'parent' if line[user_ident] == '1' else u'subject'
    else:
        user_ident = u'subject'
    adi_qrun = store.create_entity('QuestionnaireRun', identifier=u'%s_%s' % (label, subj_id),
                                   user_ident=user_ident,
                                   # XXX datetime
                                   iteration=1, completed=True, valid=valid,
                                   instance_of=questionnaire_eid)
    for eid in studies_eids:
        store.relate(adi_qrun.eid, 'related_study', eid)
    store.relate(adi_qrun.eid, 'concerns', subject_eid, subjtype='QuestionnaireRun')
    store.relate(assessment.eid, 'generates', adi_qrun.eid, subjtype='Assessment')
    for ind, name in answers_listing:
        # Always create answer, even if without value
        score_value = store.create_entity('Answer', value=float(line[ind]) if line[ind] else None,
                                          # XXX datetime
                                          question=questions[name],
                                          questionnaire_run=adi_qrun.eid)


###############################################################################
### IMAGES ####################################################################
###############################################################################
def import_images(store, pdf_eid, subject_eid, center_eid, study_eid, abide_study, images_dict, subj_id, label):
    assessment = store.create_entity('Assessment', identifier=u'%s_%s' % (label, subj_id),
                                     # XXX datetime, timepoint
                                     protocol=u'ABIDE')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    store.relate(assessment.eid, 'external_resources', pdf_eid)
    image_path = images_dict.get(subj_id)
    if image_path:
        scan_data, mri_data = {}, {}
        if label == 'anatomy':
            mri_data['sequence'] = u'T1'
            scan_data['label'] = u'anatomy'
            scan_data['type'] = u'normalized T1'
        else: # Rest
            mri_data['sequence'] = u'resting-state'
            scan_data['label'] = u'resting-state'
            scan_data['type'] = u'resting-state'
        mri_data.update(get_image_info(image_path))
        scan_data['filepath'] = osp.relpath(image_path)
        # Data properties
        scan_data['identifier'] = u'%s_%s' % (label, subj_id)
        scan_data['format'] = u'nii.gz'
        scan_data['completed'] = True
        scan_data['valid'] = True
        # Create entity
        mri_data = store.create_entity('MRIData', **mri_data)
        scan_data['has_data'] = mri_data.eid
        scan_data['related_study'] = study_eid
        scan_data = store.create_entity('Scan', **scan_data)
        store.relate(scan_data.eid, 'concerns', subject_eid, subjtype='Scan')
        store.relate(scan_data.eid, 'other_studies', abide_study, subjtype='Scan')
        # XXX device
        #store.relate(scan_data.eid, 'uses_device', device_id)
        store.relate(assessment.eid, 'generates', scan_data.eid, subjtype='Assessment')




###############################################################################
### STUDIES/CENTERS/INVESTIGATORS #############################################
###############################################################################
def create_abide_study(store):
    # Create the Abide study
    abide_study_eid = store.create_entity('Study', name=u'Abide',
                                          data_filepath=unicode(base_path),
                                          keywords=u'Abide; ASD',
                                          description=u'''The complexity and heterogeneity of ASD make it necessary to obtain large-scale samples which are difficult to attain in any individual lab. In response, the Autism Brain Imaging Data Exchange (ABIDE) hereby provides previously collected resting state functional magnetic resonance imaging (R-fMRI) datasets from 539 individuals with ASD and 573 typical controls for the purpose of data sharing in the broader scientific community. This grass-root initiative involved 16 international sites, sharing 20 samples yielding 1112 datasets composed of both MRI data and an extensive array of phenotypic information common across nearly all sites (see below). This unprecedented effort is expected to facilitate discovery science and comparisons across samples. In accordance with HIPAA guidelines and 1000 Functional Connectomes Project / INDI protocols, all datasets are anonymous, with no protected health information included. See fcon_1000.projects.nitrc.org/indi/abide''').eid
    theme_eid = store.create_entity('Theme', name=u'Autism spectrum disorders (ASD)',
                                    description=u'''Autism spectrum disorders (ASD) are characterized by qualitative impairment in reciprocal social communication, as well as by repetitive, restricted, and stereotyped behaviors. Previously considered rare, ASD are now recognized to occur in more than 1% of children, causing immense suffering to individuals and their families.''').eid
    store.relate(abide_study_eid, 'themes', theme_eid)
    return abide_study_eid

# XXX Centers/Investigators ?

def create_devices(store):
    devices = {}
    devices['anat_CMU'] = store.create_entity('Device', name=u'SIEMENS MAGNETOM Verio syngo MR B17',
                                              manufacturer=u'SIEMENS', model=u'MAGNETOM Verio').eid
    # configurations/hosted_by

###############################################################################
### MAIN ######################################################################
###############################################################################
if __name__ == '__main__':
    base_path = osp.abspath(sys.argv[4])
    store = Store(session)
    count_all_subject = 0
    # Create groups and categories
    groups, categories = create_groups_and_categories(session)
    # Create score defs
    score_defs = create_score_defs(session)
    # Questionnaire
    adi_questionnaire, adi_questions = create_adi_questionnaire(store)
    ados_questionnaire, ados_questions = create_ados_questionnaire(store)
    srs_questionnaire, srs_questions = create_srs_questionnaire(store)
    vineland_questionnaire, vineland_questions = create_vineland_questionnaire(store)
    wisc_iv_questionnaire, wisc_iv_questions = create_wisc_iv_questionnaire(store)
    # Create the Abide study
    abide_study_eid = create_abide_study(store)
    # Studies
    studies = {}
    # Centers
    centers = {}
    # Get all image files
    mprages = {}
    for img_path in glob.iglob(os.path.join(base_path, '*/*/session_1/anat_1/mprage.nii.gz')):
        subj_id =  str(int(img_path.split('/session_1/anat_1')[0].split('/')[-1]))
        mprages[subj_id] = img_path
    rests = {}
    for img_path in glob.iglob(os.path.join(base_path, '*/*/session_1/rest_1/rest.nii.gz')):
        subj_id =  str(int(img_path.split('/session_1/rest_1')[0].split('/')[-1]))
        rests[subj_id] = img_path
    # Import all subjects
    study_csv_file = osp.join(base_path, 'Phenotypic_V1_0b.csv')
    fobj = open(study_csv_file, 'rU')
    csvr = csv.reader(fobj, delimiter=',', dialect=csv.excel_tab)
    # Create subjects
    subjects = {}
    anat_pdf = {}
    rest_pdf = {}
    # IMPORT ##################################################################
    for ind_line, line in enumerate(csvr):
        if ind_line == 0:
            # Skip first line
            continue
        # SUBJECTS ############################################################
        print line[:15]
        subj_id = unicode(line[1])
        subject_eid = store.create_entity('Subject', identifier=subj_id,
                                          age=int(round(float(line[4]))),
                                          gender=SEX.get(line[5]),
                                          handedness=HANDEDNESS.get(line[6], u'unknown')).eid
        subjects[subj_id] = subject_eid
        # Related studies
        store.relate(subject_eid, 'related_studies', abide_study_eid)
        if line[0] in studies:
            store.relate(subject_eid, 'related_studies', studies[line[0]])
            anat_pdf_eid = anat_pdf.get(line[0])
            rest_pdf_eid = anat_pdf.get(line[0])
        else:
            study_eid = store.create_entity('Study', name=u'Abide - %s' % line[0],
                                            data_filepath=unicode(base_path)).eid
            studies[line[0]] = study_eid
            store.relate(subject_eid, 'related_studies', study_eid)
            # Create/Get files
            anat_pdf_eid = store.create_entity('ExternalResource', name=u'Abide - %s - Anat pdf' % line[0],
                                               related_study=study_eid,
                                               filepath=osp.relpath(u'%s_anat.pdf' % line[0])).eid
            anat_pdf[line[0]] = anat_pdf_eid
            rest_pdf_eid = store.create_entity('ExternalResource', name=u'Abide - %s - Rest pdf' % line[0],
                                               related_study=study_eid,
                                               filepath=osp.relpath(u'%s_rest.pdf' % line[0])).eid
            rest_pdf[line[0]] = rest_pdf_eid
        # Center
        if line[0] in centers:
            center_eid = centers[line[0]]
        else:
            center_eid = store.create_entity('Center', identifier=unicode(line[0]),
                                            name=unicode(line[0])).eid
            centers[line[0]] = center_eid
        # GROUPS/CATEGORIES ###################################################
        store.relate(subject_eid, 'related_groups', groups[int(line[2])])
        if line[3] in categories:
            store.relate(subject_eid, 'related_groups', categories[line[3]])
        else:
            store.relate(subject_eid, 'related_groups', categories['u'])
        # Numerical scores
        for ind, label in ((2, 'dx_group'),
                           (3, 'dsm_iv_tr'),
                           (7, 'handedness_score'),
                           (36, 'scq_total'),
                           (37, 'aq_total'),
                           (39, 'current_med_status'),
                           (41, 'off_stimulants_at_scan'),
                           (71, 'eye_status_at_scan'),
                           (72, 'age_at_mprage'),
                           (73, 'bmi'),
                           ):
            if line[ind]:
                try:
                    v = float(line[ind])
                    score_value = store.create_entity('ScoreValue',
                                                      definition=score_defs[label],
                                                      value=v).eid
                    store.relate(subject_eid, 'related_infos', score_value)
                except:
                    continue
        # String scores
        # XXX Use questionnaire for medication name ?
        for ind, label in ((38, 'comorbidity'),
                           (40, 'medication_name'),):
            if line[ind]:
                score_value = store.create_entity('ScoreValue',
                                                  definition=score_defs[label],
                                                  text=line[ind]).eid
                store.relate(subject_eid, 'related_infos', score_value)
        # FIQ/VIQ/PIQ SCORES ##################################################
        for vind, tind, label in ((8, 11, 'fiq'), (9, 12, 'viq'), (10, 13, 'piq')):
            if line[tind] and line[vind]:
                definition = score_defs.get('%s_%s' % (label, line[tind].lower()))
                if definition:
                    score_value = store.create_entity('ScoreValue',
                                                      definition=definition,
                                                      value=line[vind]).eid
                    store.relate(subject_eid, 'related_infos', score_value)
                else:
                    print 'Unknown ', label, line[vind], line[tind]
        # ADI SCORES ##########################################################
        adi_answers_listing = ((14, 'adi_r_social_total_a'),
                               (15, 'adi_r_verbal_total_bv'),
                               (16, 'adi_r_rrb_total_c'),
                               (17, 'adi_r_onset_total_d'),
                               (18, 'adi_r_rsrch_reliable'))
        import_questionnaire(store, subject_eid, center_eid, (study_eid, abide_study_eid),
                             adi_questionnaire,
                             adi_questions, adi_answers_listing, subj_id, 'adi', 18)
        # ADOS SCORES ##########################################################
        ados_answers_listing = ((19, 'ados_module'),
                                (20, 'ados_total'),
                                (21, 'ados_comm'),
                                (22, 'ados_social'),
                                (23, 'ados_stereo_behav'),
                                (24, 'ados_rsrch_reliable'),
                                (25, 'ados_gotham_soc_affect'),
                                (26, 'ados_gotham_rrb'),
                                (27, 'ados_gotham_total'),
                                (28, 'ados_gotham_severity'),)
        import_questionnaire(store, subject_eid, center_eid, (study_eid, abide_study_eid),
                             ados_questionnaire,
                             ados_questions, ados_answers_listing, subj_id, 'ados', 24)
        # SRS SCORES ##########################################################
        srs_answers_listing = ((29, 'srs_version'),
                               (30, 'srs_raw_total'),
                               (31, 'srs_awareness'),
                               (32, 'srs_cognition'),
                               (33, 'srs_communication'),
                               (34, 'srs_motivation'),
                               (35, 'srs_mannerisms'),)
        import_questionnaire(store, subject_eid, center_eid, (study_eid, abide_study_eid),
                             srs_questionnaire,
                             srs_questions, srs_answers_listing, subj_id, 'srs', None)
        # VINELAND SCORES ##########################################################
        vineland_answers_listing = ((42, 'vineland_receptive_v_scaled'),
                                    (43, 'vineland_expressive_v_scaled'),
                                    (44, 'vineland_written_v_scaled'),
                                    (45, 'vineland_communication_standard'),
                                    (46, 'vineland_personal_v_scaled'),
                                    (47, 'vineland_domestic_v_scaled'),
                                    (48, 'vineland_community_v_scaled'),
                                    (49, 'vineland_dailylvng_standard'),
                                    (50, 'vineland_interpersonal_v_scaled'),
                                    (51, 'vineland_play_v_scaled'),
                                    (52, 'vineland_coping_v_scaled'),
                                    (53, 'vineland_social_standard'),
                                    (54, 'vineland_sum_scores'),
                                    (55, 'vineland_abc_standard'),
                                    (56, 'vineland_informant'),)
        import_questionnaire(store, subject_eid, center_eid, (study_eid, abide_study_eid),
                             vineland_questionnaire,
                             vineland_questions, vineland_answers_listing, subj_id, 'vineland', None, 56)
        # WISC_IV SCORES ##########################################################
        wisc_iv_answers_listing = ((57, 'wisc_iv_vci'),
                                   (58, 'wisc_iv_pri'),
                                   (59, 'wisc_iv_wmi'),
                                   (60, 'wisc_iv_psi'),
                                   (61, 'wisc_iv_sim_scaled'),
                                   (62, 'wisc_iv_vocab_scaled'),
                                   (63, 'wisc_iv_info_scaled'),
                                   (64, 'wisc_iv_blk_dsn_scaled'),
                                   (65, 'wisc_iv_pic_con_scaled'),
                                   (66, 'wisc_iv_matrix_scaled'),
                                   (67, 'wisc_iv_digit_span_scaled'),
                                   (68, 'wisc_iv_let_num_scaled'),
                                   (69, 'wisc_iv_coding_scaled'),
                                   (70, 'wisc_iv_sym_scaled'),)
        import_questionnaire(store, subject_eid, center_eid, (study_eid, abide_study_eid),
                             wisc_iv_questionnaire,
                             wisc_iv_questions, wisc_iv_answers_listing, subj_id, 'wisc_iv', None, None)
        # IMAGES ##############################################################
        import_images(store, anat_pdf_eid, subject_eid, center_eid, study_eid,
                      abide_study_eid, mprages, subj_id, 'anatomy')
        import_images(store, rest_pdf_eid, subject_eid, center_eid, study_eid,
                      abide_study_eid, rests, subj_id, 'resting-state')


    # Flush and commit
    store.flush()
    store.commit()



# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import csv
import json
import pickle
import re

import nibabel as nb


def get_image_info(image_path, get_tr=True):
    """Return the image info readed from the image using Nibabel"""
    img = nb.load(image_path)
    data = {}
    data['voxel_res_x'] = float(img.get_header()['pixdim'][1])
    data['voxel_res_y'] = float(img.get_header()['pixdim'][2])
    data['voxel_res_z'] = float(img.get_header()['pixdim'][3])
    data['shape_x'] = int(img.get_shape()[0])
    data['shape_y'] = int(img.get_shape()[1])
    data['shape_z'] = int(img.get_shape()[2])
    data['shape_t'] = int(img.get_shape()[3]) if len(img.get_shape()) == 4 else None
    data['affine'] = pickle.dumps(img.get_affine().tolist())
    desc = str(img.get_header()['descrip'])
    # Use desc ?
    try:
        if get_tr:
            tr, te = re.findall(
                'TR=(.*)ms.*TE=(.*)ms', desc)[0]
            data['tr'] = float(tr)
            data['te'] = float(te)
    except Exception, e:
        data['tr'] = None
        data['te'] = None

    return data

def import_genes(ref_chr_path, ref_gene_path):
    """Import genes"""
    chromosomes = json.load(open(ref_chr_path))
    ref_gene = []
    for row in csv.reader(open(ref_gene_path), delimiter='\t'):
        gene = {}
        gene['name'] = unicode(row[0])
        gene['gene_id'] = unicode(row[0])
        gene['uri'] = None
        gene['start_position'] = int(row[1])
        gene['stop_position'] = int(row[2])
        gene['chromosome'] = row[3].split('_')[0]
        ref_gene.append(gene)
    return ref_gene

def import_chromosomes(ref_chr_path):
    """Import chromosomes"""
    chromosomes = json.load(open(ref_chr_path))
    chrs = []
    for chr_id in chromosomes:
        chr = {}
        chr['name'] = u'chr%s' % chromosomes[chr_id].upper()
        chr['identifier'] = unicode(chr['name'])
        chrs.append(chr)
    return chrs

def import_snps(ref_chr_path, ref_snp_path):
    """Import snps"""
    chromosomes = json.load(open(ref_chr_path))
    snps = []
    for row in csv.reader(open(ref_snp_path), delimiter='\t'):
        snp = {}
        if row[0] == '0':
            continue
        chr_id = chromosomes[row[0]]
        snp['rs_id'] = unicode(row[1])
        snp['position'] = int(row[3])
        snp['chromosome'] = u'chr%s' % chr_id.upper()
        snps.append(snp)
    return snps



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
from logilab.mtconverter import xml_escape

from cubicweb.predicates import is_instance
from cubicweb.view import EntityView
from cubicweb.web.views.baseviews import ListView


###############################################################################
### LIST VIEW #################################################################
###############################################################################
class BrainomicsListView(ListView):

    def call(self, klass=None, title=None, subvid=None, listid=None, **kwargs):
        super(BrainomicsListView, self).call(klass="unstyled stripped",
                                             title=title, subvid=subvid,
                                             listid=listid, **kwargs)


###############################################################################
### CARD VIEW #################################################################
###############################################################################
class BrainomicsCardDocumentation(EntityView):
    __regid__ = 'documentation_card'
    __select__ = EntityView.__select__ & is_instance('Card')

    def call(self, rset=None, **kwargs):
        rset = rset or self.cw_rset
        w = self.w
        w(u'<div>')
        for entity in rset.entities():
            w(u'<h2>%s</h2>' % xml_escape(entity.title))
            if entity.synopsis:
                w(u'<blockquote><p>%s</p></blockquote>' % xml_escape(entity.synopsis))
            if entity.content:
                w(entity.content)
        w(u'</div>')


###############################################################################
### RESOURCES VIEW ############################################################
###############################################################################
class BrainomicsResultFile(EntityView):
    __regid__ = 'results-view'
    __select__ = EntityView.__select__ & is_instance('Questionnaire', 'QuestionnaireRun',
                                                     'GenomicMeasure', 'Scan')

    def entity_call(self, entity):
        # Display results file
        w = self.w
        rset = self._cw.execute('Any F WHERE X results_file F, X eid %(e)s',
                                {'e': entity.eid})
        if rset:
            w(u'<h3>%s</h3>' % self._cw._('Results file'))
            self.wview('list', rset=rset)


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, (BrainomicsListView,))
    vreg.register_and_replace(BrainomicsListView, ListView)




# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-brainomics views/forms/actions/components for web ui"""

from cubicweb.web import formfields as ff, Redirect
import cubicweb.web.formwidgets as fwdgs
from cubicweb.view import StartupView
from cubicweb.web.views.basecontrollers import ViewController
from cubicweb.web.views import forms

BROWSABLE_ENTITIES = ('Subject', 'Scan')

class BrainomicsDisplaySpecificEntities(StartupView):
    __regid__ = 'display-specific-entities'

    def call(self):
        self.w(u'<div class="span12">')
        self.w(u'<h2>Display specific entities</h2>')
        self.w(u'<legend>%s</legend>' % self._cw._('Browse for identifiers file'))
        entities_form = self._cw.vreg['forms'].select('browse-identifiers-form', self._cw)
        entities_form.render(w=self.w)
        self.w(u'</div>')


class BrainomicsDisplaySpecificEntitiesForm(forms.FieldsForm):
    __regid__ = 'browse-identifiers-form'
    form_buttons = [fwdgs.SubmitButton(label=_('Validate'), attrs={"class": "btn btn-primary"})]


    @property
    def action(self):
        return self._cw.build_url('browse-identifiers-controller')

    filename = ff.FileField(name='__file',
                            label=_('Indentifiers file to import'),
                            order=1,)

    etype = ff.StringField(name='__etype',
                           label=_('Entity type'),
                           required=True,
                           help=_('Entity type'),
                           order = 2,
                           choices = BROWSABLE_ENTITIES)


class BrainomicsDisplaySpecificEntitiesController(ViewController):
    __regid__ = 'browse-identifiers-controller'

    def _read_uuid_file(self, filename):
        try:
            fdesc = filename[1]
        except IOError as ioerr:
            sys.stderr.write('File not found')
            uuids = None
        else:
            uuids = ["'" + uuid.strip() + "'" for uuid in fdesc.readlines() if uuid.strip()]
            fdesc.close()
        return uuids

    def publish(self, rset=None):
        filename = self._cw.form['__file']
        etype = self._cw.form['__etype']
        uuids = self._read_uuid_file(filename)
        if uuids:
            rql = ('Any X WHERE X is %(etype)s, X identifier IN (%(uuids)s)'
                   % {'uuids': ', '.join(uuids), 'etype': etype})
            raise Redirect(self._cw.build_url(vid='list', rql=rql))
        raise Redirect(self._cw.base_url())



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-brainomics postcreate script, executed at instance creation time or when
the cube is added to an existing instance.

You could setup site properties or a workflow here for example.
"""

set_property('ui.site-title', 'Brainomics')

# Create cards
create_entity('Card', content_format=u'text/html', title=u'index',
              content=u"""<div class="page-header"><h1>What is it?<small> And who are we?</small></h1></div>
              <p>The BRAINOMICS project is one of the projects chosen by the Agence nationale de la recherche (ANR) in the bioinformatics call of the Investissements d'avenir program. This website is a demonstration of the project based on the Localizer data (courtesy of P. Pinel, S. Dehaene, T. Bourgeron and D. Lebihan) and allows complex query and various visualization of the data.
This demo is for now only accessible on the Neurospin network.</p>
<p>This prototype was developed by V. Michel, Y. Schwartz, D. Papadopoulos and V. Frouin</p>
<p><span class="badge badge-important">This website is still in beta. Comments are welcome!</span></p>
<p> <strong>Feel free to play with it! Use the RQL <i>Search</i> field in the bar at the top of the page.</strong></p>""")

create_entity('Card', content_format=u'text/html', title=u'rqlexamples',
              content=u'''<ul>
              <li>Query all the subjects of the database
              <pre>Any S WHERE S is Subject</pre></li>
              <li>Query all right-handed subjects of the database
              <pre>Any S WHERE S is Subject, S handedness "right"</pre></li>
              <li>Query all the right-handed female subjects of the database
              <pre>Any S WHERE S is Subject, S handedness "right", S gender "female"</pre></li>
              <li>Query all the scans of male subjects
              <pre>Any X WHERE S is Subject, S gender "male", X is Scan, X concerns S</pre></li>
              <li>Query all female subjects that have a score greater than 4.O for the "algebre" question of the Localizer questionnary (Question number 2), and return both subject identifier and score
              <pre>Any I,V WHERE S is Subject, S identifier I, S gender "female", X is QuestionnaireRun, X concerns S, A is Answer, A questionnaire_run X, A question Q, Q text "algebre", A value V, A value > 4</pre></li>
              <li>Query 100 Cmap scans of subjects that have a score greater than 4.O for the "algebre" question of the Localizer questionnary (Question number 2)
              <pre>Any SA LIMIT 100 WHERE S is Subject, X is QuestionnaireRun, X concerns S, A is Answer, A questionnaire_run X, A question Q, Q text "algebre", A value > 4, SA is Scan, SA concerns S, SA type "c map"</pre></li>
              <li>Query all the Tmap scans for the "auditory calculation" contrast of subjects along with their Genomics data, when we have acquired data for Snps on the gene "CFB":
              <pre>Any S,SCF,FGEN WHERE S is Subject, SC is Scan, SC type "t map", SC label "auditory calculation", SC concerns S, SC filepath SCF, GEN is GenomicMeasure, GEN filepath FGEN, GEN concerns S, GEN platform P, P related_snps SN, G is Gene, G name "CFB", G start_position GSA, G stop_position GSO, SN position SP HAVING SP > GSA, SP &lt; GSO</pre></li>
              </ul>''')


create_entity('Card', content_format=u'text/html', title=u'license',
              content=u"... License text ...")



# -*- coding: utf-8 -*-

"""Classes for importing data from Imagen XNAT server

Expects XML dumps for metadata and raw images and genetic data
Find additionnal data from FU2 processings

Only the information about F-maps or contrast is kept
"""

import logging
import os
from lxml import etree
import glob
import csv
import sys

from cubicweb.dataimport import SQLGenObjectStore

from cubes.brainomics.importers.helpers import get_image_info

# XXX Other links/attributes

###############################################################################
### SOME GLOBALS ##############################################################
###############################################################################
# obsolete: use XPath and NAMESPACES instead
XNAT = '{http://nrg.wustl.edu/xnat}'
XSI = '{http://www.w3.org/2001/XMLSchema-instance}'
PSYTOOL = '{http://imagen.cea.fr/psytool}'
BEHAVIOURAL = '{http://imagen.cea.fr/behavioural}'
IMAGEN = '{http://imagen.cea.fr/imagen}'
FREESURFER = '{http://nrg.wustl.edu/fs}'
IMAGEN_fs = '{http://imagen.cea.fr/freesurfer}'

XNAT_TAGS_SUBJECT_INFOS = (
    'CANTAB',
    'CHILDHA',
    'CHILDRA',
    'Drugs',
    'Export',
    'Adolescent',
    'FMRI',
    'Series',
    'Guardian',
    'Medications',
    'PARENTRA',
    'PARENTSA',
    'Test',
)

##   1 !!! UNKNOWN EXPERIMENT ---> xnat:imageSessionData

XNAT_TAGS_SUBJECT_ADDITIONAL_INFOS = (
    'custom:cyclePhaseInfoData',
    'custom:sommeilData',
    'custom:bdnfData',
    'custom:oxyData',
    )

XNAT_TAGS_GENERIC_ASSESSMENT = (
    'dawba:clinicalRateData',
    'dawba:clinicalRate_fuData',
    'dawba:computerData',
    'dawba:computer_fuData',
    'dawba:youthData',
    'dawba:youth_fuData',
    'dawba:parent1Data',
    'dawba:parent1_fuData',
    'psytool:genData',
)
XNAT_TAGS_GENERIC_ASSESSMENT_RESOURCES_ONLY = (
    'imagen:rawPackageData',
)
XNAT_TAGS_NEUROIMAGING = (
    'xnat:mrSessionData',
)

XNAT_TAG_GENOMICS_SCORES = (
    'custom:per1_methylationData',
    'custom:oxtrGeneExpressionData',
    'custom:per2Data',
    'custom:per1_no13Data',
)

XNAT_TAGS_QUESTIONNAIRE = (
    'psytool:adsr_child_fuData',
    'psytool:audit_childData',
    'psytool:audit_child_fuData',
    'psytool:audit_parentData',
    'psytool:audit_parent_fuData',
    'psytool:audit_interview_fuData',
    'psytool:consent_fuData',
    'psytool:csi_child_fuData',
    'psytool:fbbhks_parent_fuData',
    'psytool:fu_reliabilityData',
    'psytool:fu_reliability_additionalData',
    'psytool:gateway_fu_parentData',
    'custom:iipData',
    'imagen:recruitmentInfosData',
    'imagen:imagenSubjectVariablesData',
    'psytool:espad_childData',
    'psytool:espad_child_fuData',
    'psytool:espad_parentData',
    'psytool:espad_parent_fuData',
    'psytool:espad_interview_fuData',
    'psytool:ctsData',
    'psytool:iri_child_fuData',
    'psytool:kirbyData',
    'psytool:kirby_fuData',
    'psytool:srs_parent_fuData',
    'psytool:stutt_parent_fuData',
    'psytool:leqData',
    'psytool:leq_fuData',
    'psytool:pbqData',
    'psytool:pdsData',
    'psytool:pds_fuData',
    'psytool:niData',
    'psytool:mast_parentData',
    'psytool:mast_parent_fuData',
    'psytool:neoffi_childData',
    'psytool:neoffi_child_fuData',
    'psytool:neoffi_parentData',
    'psytool:neoffi_parent_fuData',
    'psytool:pegboardData',
    'psytool:surpsData',
    'psytool:surps_fuData',
    'psytool:surps_parentData',
    'psytool:surps_parent_fuData',
    'psytool:tci_childData',
    'psytool:tci_child_fuData',
    'psytool:tci_parentData',
    'psytool:tci_parent_fuData',
    'psytool:tlfbData',
    'psytool:wiscData',
    'psytool:pbq_fuData',
    'custom:schizagenData',
    'custom:digitratioData',
    'custom:iat_cimhData',
    )

XNAT_TAGS_BEHAVIOURAL = (
    'psytool:srcData',
    'psytool:identData',
    'psytool:dotprobeData',
    'psytool:palp_v2Data',
    'behavioural:cantabTestsData',
)
XNAT_TAGS_SKIPPED = (
    'iteration',
    'language',
    'user_code_ident',
    'completed',
    'valid',
    'test_version',
    'trials',
    'imageSession_ID',
    'base_scan_type',
    'subject_ID',
    'ID',
    'project',
    'label',
    'type',
    '%stype' % XSI,
    'xsi:stype',
    'Task_type',
    'ratedate',
    'ratername',
    'relations'
)
CSV_DELIMITER = '\t'
PATHCHANGES = (('/data/xnat_private/archive/IMAGEN/arc001/processed/',
                '/neurospin/imagen/processed/'),
               ('/data/processed/',
                '/neurospin/imagen/processed/'),
               )

SCAN_TYPES = {'ADNI_MPRAGE': u'raw T1',
              'short_MPRAGE': u'raw T1',
              'behavioural:face_taskData': u'raw fMRI',
              'behavioural:gcaData': u'raw fMRI',
              'behavioural:mid_taskData': u'raw fMRI',
              'behavioural:recognition_taskData': u'raw fMRI',
              'behavioural:stop_signal_taskData': u'raw fMRI',
              'DTI': u'dti',
              'FLAIR': u'raw fMRI',
              'EPI_faces': u'raw fMRI',
              'EPI_global': u'raw fMRI',
              'EPI_rest': u'raw fMRI',
              'EPI_short_MID': u'raw fMRI',
              'EPI_stop_signal': u'raw fMRI',
              'T2': u'raw fMRI',
              'spm:betaData': u'preprocessed fMRI',
              'spm:beta_meanData': u'preprocessed fMRI',
              'spm:beta_mean_mvtroiData': u'preprocessed fMRI',
              'spm:cData': u'c map',
              'spm:conData': u'preprocessed fMRI',
              'spm:con_mvtroiData': u'preprocessed fMRI',
              'spm:essData': u'preprocessed fMRI',
              'spm:job_statsintraData': u'preprocessed fMRI',
              'spm:maskData': u'boolean mask',
              'spm:meanaData': u'preprocessed fMRI',
              'spm:meana_matData': u'preprocessed fMRI',
              'spm:mmprageData': u'preprocessed fMRI',
              'spm:mwcData': u'preprocessed fMRI',
              'spm:resmsData': u'preprocessed fMRI',
              'spm:rpl_aData': u'preprocessed fMRI',
              'spm:rp_txtData': u'preprocessed fMRI',
              'spm:rpvData': u'preprocessed fMRI',
              'spm:spmfData': u'preprocessed fMRI',
              'spm:spmtData': u'preprocessed fMRI',
              'spm:statsintra_mat_Data': u'preprocessed fMRI',
              'spm:wcData': u'preprocessed fMRI',
              'spm:weaData': u'preprocessed fMRI',
              'spm:weamaskData': u'preprocessed fMRI',
              'spm:wmmprageData': u'normalized T1',
              'spm:y_mprageData': u'normalized T1'}

FREESURFER_TYPES = ('fs:fsData',)


###############################################################################
### UTILITY FUNCTIONS #########################################################
###############################################################################


class ImagenException(Exception):
    """Errors encountered while parsing XML files"""

    def __init__(self, message, filename):
        Exception.__init__(self, message)
        self.filename = filename


def get_node_text(node, path, converter=None):
    """Function returning the text of the first node given a path."""
    _node = node.findall(path)
    if _node:
        info = _node[0].text
        if converter:
            try:
                return converter(info)
            except:
                return None
        return unicode(info)
    return None

def format_filepath(filepath):
    """Correctly change the path, according to some user-defined conversions
    """
    if not filepath:
        return None
    for old, new in PATHCHANGES:
        filepath = filepath.replace(old, new)
    return unicode(filepath)

def iterate_external_resources(store, experiment, study_eid):
    """Iterate over all the possible external resources (resouce/file)
    from an experiment node
    """
    added_files = set()
    for resource in experiment.findall('%(a)sresources/%(a)sresource' % {'a': XNAT}):
        filepath = format_filepath(resource.get('URI'))
        if filepath and filepath not in added_files:
            added_files.add(filepath)
            yield {'name': resource.get('label'), 'filepath': filepath, 'related_study': study_eid}
    # From Files
    for resource in experiment.findall('%(a)sFiles/%(a)sfile' % {'a': IMAGEN}):
        filepath = format_filepath(resource.get('file_name'))
        if filepath and filepath not in added_files:
            added_files.add(filepath)
            yield {'name': filepath, 'filepath': filepath, 'related_study': study_eid}

def iterate_investigators(node):
    """Iterate over the investigators of a node"""
    for assistant in node.findall('%sResearchAssistant' % IMAGEN):
        lastname = unicode(assistant.get('LastName'))
        # Cleanup lastname
        lastname = lastname.replace('-', '').strip()
        if lastname:
            yield {'identifier': lastname, 'lastname': lastname}

def iterate_score_value_infos(store, node, category, base_tag=XNAT):
    """Iterate possible score values over a node
    """
    scores = []
    try:
        node_name = node.tag.replace(base_tag, '')
    except AttributeError:
        return
    if node.text and node.text.strip():
        if node_name not in XNAT_TAGS_SKIPPED:
            scores.append((node_name, node.text.strip()))
    else:
        # Use attributes
        scores.extend((node_name + '_' + key, value) for key, value in node.items()
                      if key not in XNAT_TAGS_SKIPPED)
    for name, value in scores:
        yield build_score_value_infos(store, name, value, category)

def build_score_value_infos(store, name, value, category=u'various',
                            unit=None, possible_values=None):
    """Build score value infos from a name and a value
    """
    try:
        _value = float(value)
        _type = 'numerical'
    except:
        _type = 'string'
    if name in SCORE_DEFS:
        entity_eid = SCORE_DEFS[name]
    else:
        entity = store.create_entity('ScoreDefinition', name=name,
                                     category=category, type=_type,
                                     unit=unit, possible_values=possible_values)
        SCORE_DEFS[name] = entity.eid
        entity_eid = entity.eid
    attrs = {'definition': entity_eid}
    if _type == 'numerical':
        attrs['value'] = value
    else:
        # Truncate text
        if len(value)>=2040:
            attrs['text'] = value[:2040] + u'[...]'
        else:
            attrs['text'] = value
    return attrs

def build_csv_external_resource(store, file_id, all_scores, study_eid):
    all_data = []
    all_keys = set()
    # Two steps to merge possibly different keys
    for data in all_scores:
        all_keys.update(data.keys())
        all_data.append(data)
    all_keys = sorted(list(all_keys))
    if all_data:
        # Only create file if some data exist
        filename = os.path.join(OUTPUT_FOLDER, file_id) + '.csv'
        fobj = open(filename, 'w')
        csvwriter = csv.writer(fobj, delimiter=CSV_DELIMITER)
        # Two steps to merge possibly different keys
        csvwriter.writerow(all_keys)
        for data in all_data:
            csvwriter.writerow([data.get(key) for key in all_keys])
        fobj.close()
        extres_eid = store.create_entity('ExternalResource', name=file_id,
                                         related_study=study_eid,
                                         filepath=format_filepath(filename)).eid
        return extres_eid
    return None


###############################################################################
### GENOMICS EXPERIMENT #######################################################
###############################################################################
def import_genomics_scores(store, tree, experiment, study_eid, subject_eid, center_eid):
    """Import genomics scores
    """
    # Create assessment
    measure_eid = import_generic_test_run(store, experiment, study_eid, subject_eid,
                                          _type=u'genomics')
    assessment_eid = create_assessment_for_experiment(store, experiment, None,
                                                      study_eid, subject_eid,
                                                      center_eid, measure_eid)
    # Create measure

###############################################################################
### NEUROIMAGING EXPERIMENT ###################################################
###############################################################################
def build_scan_map(tree, scan, experiment):
    """Retrieve Scan attributes from an assessor and a scan

    Returns a map of Scan attributes, ready to be fed to the store
    """
    scan_id = unicode(scan.get('ID'))
    scan_type = unicode(scan.get('type'))
    for assessor_file in experiment.findall('%(a)sassessors/%(a)sassessor' % {'a': XNAT}):
        if assessor_file.get('type') == scan_type:
            _file = assessor_file.find('%(a)sout/%(a)sfile' % {'a': XNAT})
            break
    else:
        # No data file path
        print 'NO DATA FILEPATH FOR', scan_id, scan_type
        return
    if _file is None:
        return
    file_uri = format_filepath(_file.get('URI'))
    file_format = unicode(_file.get('format'))
    _type = SCAN_TYPES.get(scan_type, 'raw fMRI')
    attributes = {
        'identifier': scan_id, # required attribute
        'label': scan_type, # required attribute ### infer from type?
        'type': _type, # required attribute
        'format': file_format,
        'filepath': file_uri, # required attribute
        'position_acquisition': int(scan_id),
    }
    scan_time = scan.get('xnat:scanTime')
    if scan_time:
        attributes['timepoint'] = unicode(scan_time)
    return attributes

def build_scan_data_map(store, tree, scan):
    """Retrieve ScanData attributes from a scan

    Returns a map of ScanData attributes, ready to be fed to the store
    """
    root = tree.getroot()
    nsmap = root.nsmap
    scan_type = unicode(scan.get('%stype' % XSI).split(':')[-1])
    ### TODO fix below
    if scan_type == 'mrScanData':
        data_etype ='MRIData'
        scan_infos = {}
        path = 'xnat:parameters'
        find = scan.xpath(path, namespaces=nsmap)
        parameters = find[0]
        # MRIData
        for seq_name in ('sequence', 'scanSequence', 'seqVariant', 'acqType'):
            scan_infos.setdefault('sequence', []).append(get_node_text(parameters,
                                                                       '%s%s' % (XNAT, seq_name)))
        scan_infos['sequence'] = unicode('/'.join([s for s in scan_infos['sequence'] if s]))
        voxelres = parameters.findall('%svoxelRes' % XNAT)
        if voxelres:
            for axis in ('x', 'y', 'z'):
                scan_infos['voxel_res_%s' % axis] = float(voxelres[0].get(axis))
        fov = parameters.findall('%sfov' % XNAT)
        if fov:
            for axis in ('x', 'y'):
                scan_infos['fov_%s' % axis] = float(fov[0].get(axis))
        for attr in ('tr', 'te'):
            scan_infos['%s' % attr] = get_node_text(parameters, '%s%s' % (XNAT, attr), float)
    else:
        print 'Data type %s not implemented' % scan_type
        raise
    return store.create_entity(data_etype, **scan_infos).eid

def build_resource_scan_infos(resource, assessor):
    """Build scan infos from an assessor and a resource
    """
    if unicode(assessor.get('base_scan_type')):
        label = u'%s_%s' % (unicode(assessor.get('base_scan_type')), assessor.get('%stype' % XSI))
    else:
        label = assessor.get('%stype' % XSI)
    return {'format': unicode(resource.get('format')),
            'identifier': unicode(resource.get('label')),
            'label': label,
            'type': SCAN_TYPES.get(assessor.get('%stype' % XSI), 'raw fMRI'),
            'filepath': format_filepath(resource.get('URI'))}

def import_neuroimaging_raw_scan(store, tree, experiment, scan,
                                 assessment_eid, study_eid, subject_eid,
                                 center_eid, device_eid):
    """Import a raw scan with all the related entities
    """
    # Classical case - Iterate over scans
    if not device_eid:
        device_eid = import_device(store, tree, experiment, center_eid)
    scan_infos = build_scan_map(tree, scan, experiment)
    if not scan_infos:
        return
    scan_infos['concerns'] = subject_eid
    scan_infos['related_study'] = study_eid
    if device_eid:
        scan_infos['uses_device'] = device_eid
    # Add data
    scan_infos['has_data'] = build_scan_data_map(store, tree, scan)
    # Create entity
    if not scan_infos.get('label') or not scan_infos.get('filepath'):
        return
    scan_eid = store.create_entity('Scan', **scan_infos).eid
    store.relate(assessment_eid, 'generates', scan_eid, subjtype='Assessment')
    # Add external resource
    filepath = scan.findall('%sfile' % XNAT)[0]
    if format_filepath(filepath.get('URI')):
        extres_eid = store.create_entity('ExternalResource',
                                         name=unicode(filepath.get('content')),
                                         filepath=format_filepath(filepath.get('URI'))).eid
        store.relate(scan_eid, 'external_resources', extres_eid)
    return scan_infos['type'], scan_eid

def build_freesurfer_analysis(store, tree, assessor, study_eid, subject_eid):
    _type = u'Freesurfer'
    if _type in GENERIC_TESTS:
        test_eid = GENERIC_TESTS[_type]
    else:
        test = store.create_entity('GenericTest', identifier=_type,
                                   name=_type, type=_type)
        GENERIC_TESTS[_type] = test.eid
        test_eid = test.eid
    # GenericTestRun
    measure = store.create_entity('GenericTestRun',
                                  user_ident=u'child',
                                  identifier=unicode(assessor.get('ID')),
                                  instance_of=test_eid,
                                  related_study=study_eid,
                                  concerns=subject_eid)
    # Add files
    for extres_infos in iterate_external_resources(store, assessor, study_eid):
        if extres_infos.get('filepath'):
            extres_eid = store.create_entity('ExternalResource', **extres_infos).eid
            store.relate(measure.eid, 'external_resources', extres_eid)
    # Add scores - Volumetric
    volumetrics = assessor.find('%(fs)smeasures/%(fs)svolumetric' % {'fs': FREESURFER})
    if volumetrics is not None:
        for volumetric in volumetrics.getchildren():
            if volumetric.tag != '%sregions' % FREESURFER:
                name = volumetric.tag.split('}')[-1]
                value = volumetric.text.strip()
                if value and name:
                    attrs = build_score_value_infos(store, name, value, u'Freesurfer - Volumetric')
                    attrs['measure'] = measure.eid
                    score_eid = store.create_entity('ScoreValue', **attrs).eid
            else:
                # Regions
                for region in volumetric.getchildren():
                    name = region.get('name')
                    for region_score in region.getchildren():
                        if isinstance(region_score, etree._Comment):
                            continue
                        value = region_score.text.strip()
                        _name = '%s - %s' % (name, region_score.tag.split('}')[-1])
                        if value and _name:
                            attrs = build_score_value_infos(store, _name, value, u'Freesurfer - Volumetric')
                            attrs['measure'] = measure.eid
                            score_eid = store.create_entity('ScoreValue', **attrs).eid
    # Add scores - Surface
    surfaces = assessor.find('%(fs)smeasures/%(fs)ssurface' % {'fs': FREESURFER})
    if surfaces is not None:
        for hemisphere in surfaces.getchildren():
            for surface in hemisphere.getchildren():
                if isinstance(surface, etree._Comment):
                    continue
                if surface.tag != '%sregions' % FREESURFER:
                    name = hemisphere.get('name') + '_' + surface.tag.split('}')[-1]
                    value = surface.text.strip()
                    if value and name:
                        attrs = build_score_value_infos(store, name, value, u'Freesurfer - Surface')
                        attrs['measure'] = measure.eid
                        score_eid = store.create_entity('ScoreValue', **attrs).eid
                else:
                    # Regions
                    for region in surface.getchildren():
                        name = hemisphere.get('name') + '_' + region.get('name')
                        for region_score in region.getchildren():
                            if isinstance(region_score, etree._Comment):
                                continue
                            value = region_score.text.strip()
                            _name = '%s - %s' % (name, region_score.tag.split('}')[-1])
                            if value and _name:
                                attrs = build_score_value_infos(store, _name, value, u'Freesurfer - Surface')
                                attrs['measure'] = measure.eid
                                score_eid = store.create_entity('ScoreValue', **attrs).eid
    return measure.eid

def import_neuroimaging(store, tree, experiment, study_eid, subject_eid, center_eid):
    """Import a neuroimaging assessment from an experiment node"""
    # Create assessment
    assessment_eid = create_assessment_for_experiment(store, experiment, None, #TODO: None->age?
                                                      study_eid, subject_eid, center_eid)
    # Create device
    device_eid = import_device(store, tree, experiment, center_eid)
    # Generate data from experiment
    seen_scan_types = {}
    for scan in experiment.findall('%(a)sscans/%(a)sscan' % {'a': XNAT}):
        # Create a scan with an external resource (raw data) and a filepath (nii file)
        data_scan = import_neuroimaging_raw_scan(store, tree, experiment, scan,
                                                 assessment_eid, study_eid, subject_eid,
                                                 center_eid, device_eid)
        if data_scan:
            _type, eid = data_scan
            if _type and eid:
                seen_scan_types[_type] = eid
        else:
            _type, eid = None, None
    # Generate other scans (e.g. beta map) and measures (e.g. badrpData)
    all_scores = []
    for assessor in experiment.findall('%(a)sassessors/%(a)sassessor' % {'a': XNAT}):
        if assessor.get('type', None) in seen_scan_types:
            # This scan has already been related earlier
            continue
        if assessor.get('%stype' % XSI) in FREESURFER_TYPES:
            measure_eid = build_freesurfer_analysis(store, tree, assessor, study_eid, subject_eid)
            store.relate(assessment_eid, 'generates', measure_eid, subjtype='Assessment')
            continue
        resource = assessor.findall('%(a)sresources/%(a)sresource' % {'a': XNAT})
        if resource:
           # This is a scan file - Create it and relate it to a processing run
           assessor_scan_infos = build_resource_scan_infos(resource[0], assessor)
           assessor_scan_infos['concerns'] = subject_eid
           if assessor.get('number'):
               assessor_scan_infos['position_acquisition'] = int(assessor.get('number'))
           if not assessor_scan_infos.get('label') or not assessor_scan_infos.get('filepath'):
               continue
           assessor_scan_infos['related_study'] = study_eid
           # MRI Data - Use get_image_info ?
           mri_eid = store.create_entity('MRIData').eid
           assessor_scan_infos['has_data'] = mri_eid
           assessor_scan_eid = store.create_entity('Scan', **assessor_scan_infos).eid
           store.relate(assessment_eid, 'generates', assessor_scan_eid, subjtype='Assessment')
        else:
            # This xml node only contains scores, e.g. badrpData
            scan_eid = seen_scan_types.get(assessor.get('base_scan_type', None))
            if scan_eid:
                # There is a scan related to the measure
                pass
            else:
                # There is no scan - Create a CSV file for the whole assessment
                scores = dict(assessor.items())
                if scores:
                    all_scores.append(scores)
        # XXX MISSING BEHAVIOURAL DATA, e.g. xsi:type="behavioural:face_taskData
    # Write the CSV scores
    if all_scores:
        _id = unicode(experiment.attrib['ID'])+'_'+'image_scores'
        extres_eid = build_csv_external_resource(store, _id, all_scores, study_eid)
        if extres_eid:
            store.relate(assessment_eid, 'external_resources', extres_eid)
    return assessment_eid


###############################################################################
### BEHAVIOURAL EXPERIMENT ####################################################
###############################################################################
def iterate_behavioural_trials(experiment, xml_ns):
    """Iterate over the trials of a behavioural experiment
    """
    for trial in experiment.findall('%(s)strials/%(s)strial' % {'s': xml_ns}):
        yield dict([(key, value) for key, value in trial.items()
                    if key not in XNAT_TAGS_SKIPPED or key == 'ID'])

def import_behavioural(store, tree, experiment, study_eid, subject_eid, center_eid):
    """Import a behavioural (as a type of psytool experiment)
    """
    # Create behavioural data
    behavioural_run_eid, behavioural_eid, age = import_pystool_experiment(store, tree,
                                                                          experiment,
                                                                          subject_eid,
                                                                          'Questionnaire',
                                                                          'QuestionnaireRun')
    # Create questions/answers
    build_question_answer_infos(store, experiment, behavioural_eid, behavioural_run_eid)
    # Create trials file in a two steps procedure to merge possibly different keys
    _id = unicode(experiment.attrib['ID'])
    extres_eid = build_csv_external_resource(store, _id,
                                             iterate_behavioural_trials(experiment, PSYTOOL),
                                             study_eid)
    if extres_eid:
        store.relate(behavioural_run_eid, 'external_resources', extres_eid)
    # Files
    for extres_infos in iterate_external_resources(store, experiment, study_eid):
        if extres_infos.get('filepath'):
            extres_eid = store.create_entity('ExternalResource', **extres_infos).eid
            store.relate(behavioural_run_eid, 'external_resources', extres_eid)
    # Relate to an assessment
    assessment_eid = create_assessment_for_experiment(store, experiment, age,
                                                      study_eid, subject_eid, center_eid,
                                                      behavioural_run_eid)
    return behavioural_run_eid


###############################################################################
### QUESTIONNAIRE EXPERIMENT ##################################################
###############################################################################
def build_psytool_experiment(store, tree, experiment, etype):
    """Retrieve Experiment attributes from a Psytools experiment

    Discard all data relating to a specific experiment run, only keep
    metadata describing the experiment.

    Returns a map of Experiment attributes, ready to be fed to the store.
    """
    path = '@xsi:type'
    root = tree.getroot()
    nsmap = root.nsmap
    find = experiment.xpath(path, namespaces=nsmap)
    psytool_type = unicode(find[0].replace('psytool:', ''))
    simple_type = unicode(psytool_type.split('_')[0])
    path = 'psytool:test_version'
    find = experiment.xpath(path, namespaces=nsmap)
    version = unicode(find[0].text) if find else None
    path = 'psytool:language'
    find = experiment.xpath(path, namespaces=nsmap)
    language = unicode(find[0].text) if find else None
    if psytool_type in EXPERIMENTS:
        return EXPERIMENTS[psytool_type]
    entity = store.create_entity(etype, identifier=psytool_type,
                                 name=psytool_type, type=simple_type,
                                 version=version, language=language)
    EXPERIMENTS[psytool_type] = entity.eid
    return entity.eid

def build_psytool_experiment_run(store, tree, experiment, etype_run, subject_eid, experiment_eid):
    """Retrieve ExperimentRun attributes from a Psytools run

    Discard metadata describing the experiment, only keep date relating
    to a specific experiment run.

    Returns a map of ExperimentRun attributes, ready to be fed to the store.
    """
    path = '@ID'
    root = tree.getroot()
    nsmap = root.nsmap
    find = experiment.xpath(path, namespaces=nsmap)
    experiment_id = unicode(find[0])
    path = 'psytool:user_code_ident'
    find = experiment.xpath(path, namespaces=nsmap)
    user_ident = unicode(find[0].text) if find else u'unknown'
    path = 'psytool:iteration'
    find = experiment.xpath(path, namespaces=nsmap)
    iteration = int(find[0].text) if find else None
    path = 'psytool:completed'
    find = experiment.xpath(path, namespaces=nsmap)
    completed = bool(find[0].text) if find else None
    path = 'psytool:processed_age_for_test'
    find = experiment.xpath(path, namespaces=nsmap)
    if find:
        age = int(find[0].text)
    else:
        path = 'psytool:age_for_test'
        find = experiment.xpath(path, namespaces=nsmap)
        age = int(find[0].text) if find else None
    return store.create_entity(etype_run,
                               identifier=experiment_id,
                               iteration=iteration,
                               completed=completed,
                               user_ident=user_ident,
                               concerns=subject_eid,
                               instance_of=experiment_eid).eid, age

def import_pystool_experiment(store, tree, experiment, subject_eid, exp_etype, etype_run):
    """Import all types of psytool experiments (questionnaire/behavioural)"""
    # Create experiment metadata - if not already created by a different
    # run of the same experiment
    experiment_eid = build_psytool_experiment(store, tree, experiment, exp_etype)
    # Create experiment data specific to this experiment run, relate to
    # experiment metadata
    # Retrieve the age of subject at the time of experiment
    experiment_run_eid, age = build_psytool_experiment_run(store, tree, experiment,
                                                           etype_run,
                                                           subject_eid,
                                                           experiment_eid)
    return experiment_run_eid, experiment_eid, age

def build_question_answer_infos(store, experiment, questionnaire_eid, questionnaire_run_eid):
    """Iterate over questions and answers for a questionnaire
    """
    _type = experiment.attrib['%stype' % XSI].replace('psytool:', '').split('_')[0]
    # Construct questions from children
    ind = 0
    data = [(child.tag.split('}')[-1], child.text) for child in experiment.getchildren()]
    # Construct questions from attributes
    data.extend(experiment.items())
    for key, value in data:
        if key not in XNAT_TAGS_SKIPPED:
            try:
                value = float(value)
                possible_answers = None
                _type = u'numerical'
            except ValueError:
                # keep string as int, use possible_answers
                possible_answers = unicode(value)
                value = 0
                _type = u'text'
            # Question
            identifier = key
            if identifier in QUESTIONS:
                question_eid = QUESTIONS[identifier]
                # Update possible answer
                if possible_answers:
                    old_possible_answers = (QUESTION_POSSIBLE_ANSWERS[question_eid] or u'').split('/')
                    if possible_answers not in old_possible_answers:
                        old_possible_answers += (possible_answers,)
                        QUESTION_POSSIBLE_ANSWERS[question_eid] = '/'.join(old_possible_answers)
                    value = old_possible_answers.index(possible_answers)
            else:
                question = store.create_entity('Question', identifier=identifier,
                                               text=key, position=ind,
                                               type=_type,
                                               questionnaire=questionnaire_eid,
                                               possible_answers=possible_answers)
                QUESTIONS[identifier] = question.eid
                question_eid = question.eid
                QUESTION_POSSIBLE_ANSWERS[question_eid] = possible_answers
            # Answer
            answer = store.create_entity('Answer', value=value,
                                         questionnaire_run=questionnaire_run_eid,
                                         question=question_eid)
            ind += 1

def import_questionnaire(store, tree, experiment, study_eid, subject_eid, center_eid):
    """Import a questionnaire (as a type of psytool experiment)"""
    # Create questionnaire
    questionnaire_run_eid, questionnaire_eid, age = import_pystool_experiment(store, tree,
                                                                              experiment,
                                                                              subject_eid,
                                                                              'Questionnaire',
                                                                              'QuestionnaireRun')
    # Create questions/answers
    build_question_answer_infos(store, experiment, questionnaire_eid, questionnaire_run_eid)
    # Create external resources
    for extres_infos in iterate_external_resources(store, experiment, study_eid):
        if extres_infos.get('filepath'):
            extres_eid = store.create_entity('ExternalResource', **extres_infos).eid
            store.relate(questionnaire_run_eid, 'external_resources', extres_eid)
    # Relate to an assessment
    assessment_eid = create_assessment_for_experiment(store, experiment, age,
                                                      study_eid, subject_eid, center_eid,
                                                      questionnaire_run_eid)
    # Files
    for extres_infos in iterate_external_resources(store, experiment, study_eid):
        if extres_infos.get('filepath'):
            extres_eid = store.create_entity('ExternalResource', **extres_infos).eid
            store.relate(questionnaire_run_eid, 'external_resources', extres_eid)
    return questionnaire_run_eid


###############################################################################
### GENERIC ASSESSMENT / EXPERIMENT ###########################################
###############################################################################
def import_generic_test_run(store, experiment, study_eid, subject_eid, _type=None):
    """Import a generic measure from an experiment node
    """
    # Create the GenericTest if it does not exist
    _type = unicode(experiment.get('%stype' % XSI)) if not _type else _type
    identifier = unicode(experiment.get('%stype' % XSI))
    if identifier in GENERIC_TESTS:
        test_eid = GENERIC_TESTS[identifier]
    else:
        test = store.create_entity('GenericTest', identifier=identifier,
                                   name=identifier, type=_type)
        GENERIC_TESTS[identifier] = test.eid
        test_eid = test.eid
    # Create the GenericTestRun
    measure = store.create_entity('GenericTestRun',
                                  user_ident=u'unknown',
                                  identifier=unicode(experiment.get('ID')),
                                  instance_of=test_eid,
                                  related_study=study_eid,
                                  concerns=subject_eid)
    # Add scores
    for score_value_infos in iterate_score_value_infos(store, experiment,
                                                       _type or u'generic'):
        score_value_infos['measure'] = measure.eid
        score_value_eid = store.create_entity('ScoreValue', **score_value_infos).eid
    return measure.eid

def create_assessment_for_experiment(store, experiment, age, study_eid,
                                     subject_eid, center_eid, measure_eid=None):
    """Create an assessment and required relations from and experiment

    Arguments:
    experiment -- XML node of the experiment
    age -- age of subject at the time of expriment
    study_eid -- identifier of the study this experiment is part of
    subject_eid -- identifier of the subject
    center_eid -- indentifier of the center where the expriment is conducted
    measure_eid -- ?

    Returns the identifier of the newly created Assessment
    """
    # Create an assessment
    experiment_id = unicode(experiment.get('ID'))
    assessment_eid = store.create_entity('Assessment',
                                         identifier=experiment_id,
                                         age_for_assessment=age,
                                         protocol=unicode(experiment.get('%stype' % XSI)),
                                         related_study=study_eid).eid
    # Create relations
    store.relate(subject_eid, 'concerned_by', assessment_eid)
    store.relate(center_eid, 'holds', assessment_eid)
    if measure_eid:
        store.relate(assessment_eid, 'generates', measure_eid, subjtype='Assessment')
    # Files
    for extres_infos in iterate_external_resources(store, experiment, study_eid):
        if extres_infos.get('filepath'):
            extres_eid = store.create_entity('ExternalResource', **extres_infos).eid
            store.relate(assessment_eid, 'external_resources', extres_eid)
    # Investigators
    for investigator_infos in iterate_investigators(experiment):
        lastname = investigator_infos['lastname']
        if lastname in INVESTIGATORS:
            investigator_eid = INVESTIGATORS[lastname]
        else:
            investigator_eid = store.create_entity('Investigator', identifier=lastname,
                                                   lastname=lastname).eid
        store.relate(assessment_eid, 'conducted_by', investigator_eid)
    return assessment_eid

def import_quality_report(store, tree, experiment, study_eid, subject_eid, center_eid):
    """Import the quality report assessment from an experiment node
    """
    measure_eid = import_generic_test_run(store, experiment, study_eid, subject_eid)
    assessment_eid = create_assessment_for_experiment(store, experiment, None,  #TODO: None->age
                                                      study_eid, subject_eid,
                                                      center_eid, measure_eid)
    for _type in XNAT_TAGS_SUBJECT_INFOS:
        for child in experiment.findall('%s%s' % (IMAGEN, _type)):
            for score_value_infos in iterate_score_value_infos(store, child, u'qualityreport',
                                                               base_tag=IMAGEN):
                score_value_infos['measure'] = measure_eid
                score_value_eid = store.create_entity('ScoreValue', **score_value_infos).eid
    return assessment_eid

def import_generic_subject_assessments(store, tree, experiment, study_eid, subject_eid, center_eid):
    """Import generic assessments infos for various data of a subject,
    such as raw package, clinical rate, ... as Questionnaires
    """
    testrun_eid = import_questionnaire(store, tree, experiment, study_eid, subject_eid, center_eid)
    # Construct scores from relations - Multiples responses
    for relation in experiment.findall('%(p)srelations/%(p)srelation' % {'p': PSYTOOL}):
        # XXX More generic ? Missing information ?
        key = unicode(relation.get('relation'))
        value = unicode(relation.get('disorder'))
        if key != 'None' and value != 'None':
            score_value_infos = build_score_value_infos(store, key, value, category=u'relations')
            score_value_infos['measure'] = testrun_eid
            score_value_eid = store.create_entity('ScoreValue', **score_value_infos).eid
    return testrun_eid


###############################################################################
### MEDICAL EXP DATA ##########################################################
###############################################################################
def import_center(store, tree):
    """Retrieve Center attributes
    The schema and other functions require a Center. If data is missing
    from the XML tree to build one, create it from scratch instead of
    bailing out.

    Returns a map of Center attributes, ready to be fed to the store.
    """
    root = tree.getroot()
    nsmap = root.nsmap
    # Center id
    path = '//xnat:experiment[@xsi:type="imagen:imagenSubjectVariablesData"]/@ImagingCentreID'
    find = root.xpath(path, namespaces=nsmap)
    identifier = unicode(find[0]) if find else u''
    if not identifier:
        return
    # Center city
    path = '//xnat:experiment[@xsi:type="imagen:imagenSubjectVariablesData"]/@ImagingCentreCity'
    find = root.xpath(path, namespaces=nsmap)
    if find and identifier:
        city = unicode(find[0])
    else:
        identifier = 0
        city = u'unknown'
    name = u'%s %s' % (identifier, city)
    if identifier in CENTERS:
        return CENTERS[identifier]
    entity = store.create_entity('Center', identifier=identifier,
                                 name=name, city=city)
    CENTERS[identifier] = entity.eid
    return entity.eid

def import_study(store, tree, filepath):
    """Create Study attributes ex nihilo
    Imagen XML files lack the relevant information but it is
    straightforward to create one.

    Returns a map of Study attributes, ready to be fed to the store.
    """
    name = u'Imagen'
    if name in STUDIES:
        return STUDIES[name]
    entity = store.create_entity('Study', name=name,
                                 data_filepath=filepath,
                                 description=u'Imagen XNAT data')
    STUDIES[name] = entity.eid
    return entity.eid

def import_subject(store, tree):
    """Retrieve Subject attributes

    XPath of relevant nodes in XML file:
    - xnat:Subject
    - xnat:Subject/xnat:demographics

    The schema and other functions require a Subject. If the XML tree is
    missing relevant data, bail out and throw an ImagenException because
    it wouldn't make sense to invent a Subject.

    Returns a map of Subject attributes, ready to be fed to the store.
    """
    # ID
    root = tree.getroot()
    nsmap = root.nsmap
    identifier = unicode(root.get('ID'))
    if not identifier:
        message = 'missing subject ID'
        logger.error('{0}: {1}'.format(tree.docinfo.URL, message))
        raise ImagenException(message, tree.docinfo.URL)
    if identifier in SUBJECTS:
        return SUBJECTS[identifier]
    # gender
    find = root.find('%(a)sdemographics/%(a)sgender' % {'a': XNAT})
    if find is not None:
        gender = find.text
        vocabulary = ('male', 'female', 'unknown')
        if gender in vocabulary:
            gender = unicode(gender)
        else:
            message = 'unexpected gender {0}'.format(gender)
            logger.warn('{0}: {1}'.format(tree.docinfo.URL, message))
            gender = u'unknown'
    else:
        message = 'missing gender'
        logger.warn('{0}: {1}'.format(tree.docinfo.URL, message))
        gender = u'unknown'
    # handedness
    find = root.find('%(a)sdemographics/%(a)shandedness' % {'a': XNAT})
    if find is not None:
        handedness = find.text
        vocabulary = ('right', 'left', 'ambidextrous', 'unknown')
        if handedness in vocabulary:
            handedness = unicode(handedness)
        else:
            message = 'unexpected handedness {0}'.format(handedness)
            logger.warn('{0}: {1}'.format(tree.docinfo.URL, message))
            handedness = u'unknown'
    else:
        message = 'missing handedness'
        logger.warn('{0}: {1}'.format(tree.docinfo.URL, message))
        handedness = u'unknown'
    # Create entity
    entity = store.create_entity('Subject', identifier=identifier,
                                 gender=gender, handedness=handedness)
    SUBJECTS[identifier] = entity.eid
    return entity.eid

def import_demographics(store, tree, subject_eid):
    """Import subject demographics (excluding gender and handedness)
    """
    demographics = tree.getroot().findall('%sdemographics' % XNAT)
    if not demographics:
        return
    for child in demographics[0].getchildren():
        if child.tag not in('%sgender' % XNAT, '%shandedness' % XNAT):
            for score_value_infos in iterate_score_value_infos(store, child, u'demographics'):
                score_value_eid = store.create_entity('ScoreValue', **score_value_infos).eid
                store.relate(subject_eid, 'related_infos', score_value_eid)

def import_subject_additional_infos(store, experiment, study_eid, subject_eid, center_eid):
    """Import subject additional infos (e.g. cyclePhaseInfo)
    """
    name = experiment.get('project', u'additional_infos')
    for score_value_infos in iterate_score_value_infos(store, experiment, name):
        score_value_eid = store.create_entity('ScoreValue', **score_value_infos).eid
        store.relate(subject_eid, 'related_infos', score_value_eid)


def import_device(store, tree, experiment, center_eid):
    """Retrieve Device attributes from an experiment

    Look for scanner nodes under the experiment node.

    Returns a map of Device attributes, ready to be fed to the store.
    """
    path = 'xnat:scanner'
    root = tree.getroot()
    nsmap = root.nsmap
    find = experiment.xpath(path, namespaces=nsmap)
    if find:
        name = unicode(find[0].text)
        if name == u'None':
            name = None
    else:
        return None
    path = 'xnat:scanner/@manufacturer'
    find = experiment.xpath(path, namespaces=nsmap)
    manufacturer = unicode(find[0]) if find else None
    if not name:
        name = manufacturer
    path = 'xnat:scanner/@model'
    find = experiment.xpath(path, namespaces=nsmap)
    model = unicode(find[0]) if find else None
    if name in DEVICES:
        return DEVICES[name]
    entity = store.create_entity('Device', name=name,
                                 manufacturer=manufacturer,
                                 hosted_by=center_eid, model=model)
    DEVICES[name] = entity.eid
    return entity.eid


###############################################################################
### MAIN IMPORTER FUNCTIONS ###################################################
###############################################################################
def import_imagen_file(store, xml_file, filepath):
    # Read XML file, get the root element, parse using XPath
    tree = etree.parse(xml_file)
    root = tree.getroot()
    nsmap = root.nsmap
    # Center
    center_eid = import_center(store, tree)
    # Study
    study_eid = import_study(store, tree, filepath)
    # Subject
    subject_eid = import_subject(store, tree)
    store.relate(subject_eid, 'related_studies', study_eid)
    # Import assessments
    import_demographics(store, tree, subject_eid)
    # Import experiments
    path = '/xnat:Subject/xnat:experiments/xnat:experiment'
    find = root.xpath(path, namespaces=nsmap)
    for experiment in find:
        find = experiment.xpath('@xsi:type', namespaces=nsmap)
        experiment_type = find[0]
        if experiment_type == 'imagen:qualityReportData':
            assessment_eid = import_quality_report(store, tree, experiment,
                                                   study_eid, subject_eid, center_eid)
        elif experiment_type in XNAT_TAGS_SUBJECT_ADDITIONAL_INFOS:
            assessment_eid = import_subject_additional_infos(store, experiment,
                                                             study_eid, subject_eid, center_eid)
        elif experiment_type in XNAT_TAGS_GENERIC_ASSESSMENT_RESOURCES_ONLY:
            assessment_eid = create_assessment_for_experiment(store, experiment, None,
                                                              study_eid, subject_eid, center_eid)
        elif experiment_type in XNAT_TAGS_GENERIC_ASSESSMENT:
            testrun_eid = import_generic_subject_assessments(store, tree, experiment,
                                                             study_eid, subject_eid, center_eid)
        elif experiment_type in XNAT_TAGS_QUESTIONNAIRE:
            measure_eid = import_questionnaire(store, tree, experiment,
                                               study_eid, subject_eid, center_eid)
        elif experiment_type in XNAT_TAGS_NEUROIMAGING:
            assessment_eid = import_neuroimaging(store, tree, experiment,
                                                 study_eid, subject_eid, center_eid)
        elif experiment_type in XNAT_TAGS_BEHAVIOURAL:
            measure_eid = import_behavioural(store, tree, experiment,
                                             study_eid, subject_eid, center_eid)
        elif experiment_type in XNAT_TAG_GENOMICS_SCORES:
            measure_eid = import_genomics_scores(store, tree, experiment,
                                                 study_eid, subject_eid, center_eid)
        else:
            print '!!! UNKNOWN EXPERIMENT --->', experiment_type
            continue


###############################################################################
### ENTITIES/RELATIONS CACHE ##################################################
###############################################################################
SUBJECTS = dict(session.execute('Any I, X WHERE X is Subject, X identifier I'))
CENTERS = dict(session.execute('Any I, X WHERE X is Center, X identifier I'))
DEVICES = dict(session.execute('Any I, X WHERE X is Device, X name I'))
STUDIES = dict(session.execute('Any I, X WHERE X is Study, X name I'))
GENERIC_TESTS = dict(session.execute('Any I, X WHERE X is GenericTest, X identifier I'))
SCORE_DEFS = dict(session.execute('Any I, X WHERE X is ScoreDefinition, X name I'))
INVESTIGATORS = dict(session.execute('Any I, X WHERE X is Investigator, X identifier I'))
EXPERIMENTS = dict(session.execute('Any I, X WHERE X is IN (Questionnaire, GenericTest), X identifier I'))
QUESTIONS = dict(session.execute('Any I, X WHERE X is Question, X identifier I'))
QUESTION_POSSIBLE_ANSWERS = dict(session.execute('Any X, A WHERE X is Question, X possible_answers A'))
REGIONS = dict(session.execute('Any I, X WHERE X is AnatomicalRegion, X name I'))
PROCESSING_RUNS = dict(session.execute('Any I, X WHERE X is ProcessingRun, X name I'))
RELATED_PROCESSING = set([(e[0], e[1]) for e in session.execute('Any X, Y WHERE X related_processing Y')])
OUTPUT_FOLDER = None


###############################################################################
### RUN THE IMPORTER ##########################################################
###############################################################################
if __name__ == '__main__':
    # Logging
    logger = logging.getLogger('imagen')
    logger.setLevel(logging.DEBUG)
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    logger.addHandler(console)
    # Create store
    path = sys.argv[-2]
    OUTPUT_FOLDER = sys.argv[-1]
    store = SQLGenObjectStore(session)
    for ind, xml_file in enumerate(glob.iglob(os.path.join(path, '*.xml'))):
        print 80*'*'
        print 'Processing', xml_file
        try:
            import_imagen_file(store, xml_file, path)
        except ImagenException:
            # typically XML files with imagen:Imagen_subject
            # instead of xnat:Subject
            continue
        if ind and ind % 100 == 0:
            print 80*'*'
            print 80*'*'
            print 'FLUSHING !'
            store.flush()
            print 80*'*'
            print 80*'*'
    # Update all questions
    for eid, pa in QUESTION_POSSIBLE_ANSWERS.iteritems():
        store.rql('SET X possible_answers %(pa)s WHERE X eid %(e)s',
                  {'e': eid, 'pa': pa})
    store.flush()
    store.commit()
    # Update gender - Take gender from the QR
    rset = rql('Any S, V WHERE SD is ScoreDefinition, SD category %(c)s, SD name %(n)s, SV definition SD, '
               'SV measure M, M concerns S, SV text V',
               {'n': 'Adolescent_Gender', 'c': 'qualityreport'})
    for seid, g in rset:
        if g.lower() in ('male', 'female'):
            session.execute('SET X gender %(g)s WHERE X eid %(e)s', {'e': seid, 'g': g.lower()})
    session.commit()
